/***************************************************************************
                          thegame.h  -  description
                             -------------------
    begin                : Mit Feb 13 13:38:33 MET 2002
    copyright            : (C) 2002 by Christoph Maciejewski
    email                : chrmac@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef THEGAME_H
#define THEGAME_H

// include files for QT
#include <qapp.h>
#include <qmainwindow.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qstatusbar.h>
#include <qwhatsthis.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qmsgbox.h>
#include <qfiledialog.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qevent.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qframe.h>

// Unter Win32 heisst die Klasse: WarshipsWindow

// application specific includes
#include "gamearea.h"

/**
  * This Class is the base class for your application. It sets up the main
  * window and providing a menubar, toolbar
  * and statusbar. For the main view, an instance of class TheGameView is
  * created which creates your view.
  */
class WarshipsWindow : public QMainWindow
{
  Q_OBJECT

  public:
    /** construtor */
    WarshipsWindow(QWidget* parent = 0, const char* name = 0, WFlags f = WType_TopLevel);
    /** destructor */
    ~WarshipsWindow();
    /** initializes all QActions of the application */
    void initActions();
    /** initMenuBar creates the menu_bar and inserts the menuitems */
    void initMenuBar();
    /** setup the statusbar */
    void initStatusBar();

    void shoot();


  public slots:

    /** start a new game*/
    void slotNewGame();

    /** shows an about dlg*/
    void slotHelpAbout();

    /** shows game settings*/
    void slotSettings();

  protected:
  	void resizeEvent(QResizeEvent *r);
  	void keyPressEvent( QKeyEvent *k );

  private slots:
    void moveShot();
    void newShoot1();
    void newShoot2();
    void startShoot();
    void refreshAfterShot();
    void endOfGame(int win);
    void generateWind();
    bool connectNetwork();
    int startServer();
    int startClient();
    void reciveData();
    void sendData();
	void WSAInit(void);

  private:
  	// Game values
  	int life1, life2, ange1, ange2, speed1, speed2, proj1, proj2;
  	double ange1d, ange2d;
  	int prjN1[3], prjN2[3];
  	int who; 	// wer ist dran? 0: Spieler1, 1: Spieler2
  	int gravi;
  	double wind;
  	double tmp;
	int windArray[61];

  	// Network
  	bool network;	// it is a network game? Default: false
  	int typeN;	// 1: Server, 0: Client. Default: 1
  	char serverIP[20];	// Server IP-adress, like "127.0.0.1" (default)
  	char clientIP[20];	// Client IP-adress, like "0.0.0.0" (default)
  	bool dataOK;		// opponent game-data allready recived?


  	// Game Background - class
  	GameArea *view;

  	// Main window objects
  	QPushButton *player1;
	QPushButton *player2;

  	void shot();
    QPopupMenu *fileMenu;
    QPopupMenu *editMenu;
    QPopupMenu *helpMenu;

    QAction *fileNew;
    QAction *fileQuit;
    QAction *helpAboutApp;
    QAction *gameSet;

    int xres;
    int yres;
    int speed;
    int randomN;
    //network
    int create_socket, new_socket;

    bool gameRuns;

    //QPixmap *shotp;
    QString picPath;
    QPixmap *start;
    QPixmap *skala1;
    QLabel *startL;
    QLabel *skala1L;
    QLabel *skala2L;
    QFrame *frame1;
    QFrame *frame2;

    int xshot, yshot;
    QTimer *autoShootTimer;
    QTimer *autoRefreshTimer;

};
#endif
